#RainDrop.py
import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

import math
import random

class RainDrop(AtlanSprite):
    """
        Represents one drop of rain that falls from the cloud towards the ground.
        Makes a splash when it lands and is affected by wind.
    """

    def __init__(self, myBounds):
        """
            Place it at the top left corner of the rectangle supplied.
        """
        self.x = myBounds.x
        self.y = myBounds.y
        self.endX = myBounds.x
        self.endY = myBounds.y
        self.fallSpeed = 0.05
        self.wind = 0.0
        self.alive = True
        self.shouldRemove = False
        self.shouldExplode = True
        #each drop of rain doesnt exactly follow the wind
        self.turbulence = (random.random() - 0.5) * 0.1
        self.particles = []

    def draw(self, surface):
        """
            If it hasnt hit the ground yet, draw it as a line of length 3.
            If it has hit the ground, draw little specs for the splash.
        """
        if self.alive == True:
            pygame.draw.aaline(surface, [0, 0, 200, 100], [self.x, self.y], [self.endX, self.endY], 1)
        else:
            for p in self.particles:
                pygame.draw.aaline(surface, [0, 0, 200, 100], [p[0], p[1]], [p[0], p[1]], 1)

    def explode(self):
        """
            Do a particle effect for collision with a ground surface.
        """
        self.alive = False
        self.shouldExplode = False
        for i in xrange(5):
            self.particles.append([self.x, self.y, (random.random() - 0.5) * 0.1, -0.09])
            

    def setWind(self, w):
        """
            The velocity depends on the supplied wind and random turbulence.
        """
        self.wind = w + self.turbulence

    def update(self, delta):
        """
            Update internal logic.
        """
        #if falling, adjust position for wind
        #end is along the vector of wind but only 3 pixels in length
        if self.alive == True:
            vX = self.wind * (delta / 40.0)
            vY = delta * self.fallSpeed
            self.y += vY
            self.x += vX
            mag = math.sqrt(math.pow(vX, 2) + math.pow(vY, 2))
            vX /= mag
            vY /= mag
            self.endX = (vX * 3.0) + self.x
            self.endY = (vY * 3.0) + self.y
        else:
            #draw the particles
            #if all the particles are below ground, go ahead and remove this
            #entity completely
            allDone = True
            for p in self.particles:
                p[3] += 0.01
                p[0] += p[2] * delta
                p[1] += p[3] * delta
                if p[1] < YRES - 5:
                    allDone = False
            self.shouldRemove = allDone
        
